@@ -794,7 +794,6 @@ static const NodeImplVtbl HTMLAnchorElementImplVtbl = {
     NULL,
     NULL,
     NULL,
-    NULL,
     HTMLAnchorElement_traverse,
     HTMLAnchorElement_unlink
 };
@@ -812,7 +811,7 @@ static dispex_static_data_t HTMLAnchorElement_dispex = {
     HTMLElement_init_dispex_info
 };
 
-HRESULT HTMLAnchorElement_Create(HTMLDocumentNode *doc, nsIDOMHTMLElement *nselem, HTMLElement **elem)
+HRESULT HTMLAnchorElement_Create(HTMLDocumentNode *doc, nsIDOMElement *nselem, HTMLElement **elem)
 {
     HTMLAnchorElement *ret;
     nsresult nsres;
@@ -826,7 +825,7 @@ HRESULT HTMLAnchorElement_Create(HTMLDocumentNode *doc, nsIDOMHTMLElement *nsele
 
     HTMLElement_Init(&ret->element, doc, nselem, &HTMLAnchorElement_dispex);
 
-    nsres = nsIDOMHTMLElement_QueryInterface(nselem, &IID_nsIDOMHTMLAnchorElement, (void**)&ret->nsanchor);
+    nsres = nsIDOMElement_QueryInterface(nselem, &IID_nsIDOMHTMLAnchorElement, (void**)&ret->nsanchor);
     assert(nsres == NS_OK);
 
     *elem = &ret->element;
