@@ -43,8 +43,8 @@ typedef struct tagWND
     WNDPROC        winproc;       /* Window procedure */
     DWORD          tid;           /* Owner thread id */
     HINSTANCE      hInstance;     /* Window hInstance (from CreateWindow) */
-    RECT           rectClient;    /* Client area rel. to parent client area */
-    RECT           rectWindow;    /* Whole window rel. to parent client area */
+    RECT           client_rect;   /* Client area rel. to parent client area */
+    RECT           window_rect;   /* Whole window rel. to parent client area */
     RECT           visible_rect;  /* Visible part of the whole rect, rel. to parent client area */
     RECT           normal_rect;   /* Normal window rect saved when maximized/minimized */
     POINT          min_pos;       /* Position for minimized window */
@@ -61,7 +61,8 @@ typedef struct tagWND
     HICON          hIcon;         /* window's icon */
     HICON          hIconSmall;    /* window's small icon */
     HICON          hIconSmall2;   /* window's secondary small icon, derived from hIcon */
-    DPI_AWARENESS_CONTEXT  dpi_awareness; /* DPI awareness context */
+    UINT           dpi;           /* window DPI */
+    DPI_AWARENESS  dpi_awareness; /* DPI awareness */
     struct window_surface *surface; /* Window surface if any */
     struct tagDIALOGINFO *dlgInfo;/* Dialog additional info (dialogs only) */
     int            pixel_format;  /* Pixel format set by the graphics driver */
@@ -128,6 +129,7 @@ extern void WINPOS_ActivateOtherWindow( HWND hwnd ) DECLSPEC_HIDDEN;
 extern UINT WINPOS_MinMaximize( HWND hwnd, UINT cmd, LPRECT rect ) DECLSPEC_HIDDEN;
 extern void WINPOS_SysCommandSizeMove( HWND hwnd, WPARAM wParam ) DECLSPEC_HIDDEN;
 
+extern UINT get_monitor_dpi( HWND hwnd ) DECLSPEC_HIDDEN;
 extern BOOL set_window_pos( HWND hwnd, HWND insert_after, UINT swp_flags,
                             const RECT *window_rect, const RECT *client_rect,
                             const RECT *valid_rects ) DECLSPEC_HIDDEN;
