@@ -41,6 +41,37 @@
 static const char *publisherId = "urn:uuid:3AE5617D-790F-408A-9374-359A77F924A3";
 static const char *sequenceId = "urn:uuid:b14de351-72fc-4453-96f9-e58b0c9faf38";
 
+static const char testProbeMessage[] = "<?xml version=\"1.0\" encoding=\"utf-8\"?>"
+    "<soap:Envelope xmlns:soap=\"http://www.w3.org/2003/05/soap-envelope\" "
+    "xmlns:wsa=\"http://schemas.xmlsoap.org/ws/2004/08/addressing\" "
+    "xmlns:wsd=\"http://schemas.xmlsoap.org/ws/2005/04/discovery\" "
+    "xmlns:grog=\"http://more.tests/\"><soap:Header><wsa:To>urn:schemas-xmlsoap-org:ws:2005:04:discovery</wsa:To>"
+    "<wsa:Action>http://schemas.xmlsoap.org/ws/2005/04/discovery/Probe</wsa:Action>"
+    "<wsa:MessageID>urn:uuid:%s</wsa:MessageID>"
+    "<grog:Perry>ExtraInfo</grog:Perry></soap:Header>"
+    "<soap:Body><wsd:Probe><wsd:Types>grog:Cider</wsd:Types><grog:Lager>MoreInfo</grog:Lager></wsd:Probe></soap:Body></soap:Envelope>";
+
+static const WCHAR discoveryTo[] = {
+    'u','r','n',':',
+    's','c','h','e','m','a','s','-','x','m','l','s','o','a','p','-','o','r','g',':',
+    'w','s',':','2','0','0','5',':','0','4',':',
+    'd','i','s','c','o','v','e','r','y', 0 };
+
+static const WCHAR actionProbe[] = {
+    'h','t','t','p',':','/','/',
+    's','c','h','e','m','a','s','.','x','m','l','s','o','a','p','.','o','r','g','/',
+    'w','s','/','2','0','0','5','/','0','4','/',
+    'd','i','s','c','o','v','e','r','y','/',
+    'P','r','o','b','e', 0 };
+
+static const WCHAR uri_more_tests[] = { 'h','t','t','p',':','/','/','m','o','r','e','.','t','e','s','t','s','/', 0 };
+static const WCHAR uri_more_tests_no_slash[] = { 'h','t','t','p',':','/','/','m','o','r','e','.','t','e','s','t','s', 0 };
+static const WCHAR prefix_grog[] = { 'g','r','o','g', 0 };
+static const WCHAR name_cider[] = { 'C','i','d','e','r', 0 };
+
+static HANDLE probe_event = NULL;
+static UUID probe_message_id;
+
 #define MAX_CACHED_MESSAGES     5
 #define MAX_LISTENING_THREADS  20
 
@@ -204,7 +235,7 @@ static void start_listening(messageStorage *msgStorage, const char *multicastAdd
 {
     struct addrinfo *multicastAddr = NULL, *bindAddr = NULL, *interfaceAddr = NULL;
     listenerThreadParams *parameter = NULL;
-    const DWORD receiveTimeout = 5000;
+    const DWORD receiveTimeout = 500;
     const UINT reuseAddr = 1;
     HANDLE hThread;
     SOCKET s = 0;
@@ -241,7 +272,7 @@ static void start_listening(messageStorage *msgStorage, const char *multicastAdd
     if (multicastAddr->ai_family == AF_INET6)
         ((SOCKADDR_IN6 *)multicastAddr->ai_addr)->sin6_scope_id = 0;
 
-    /* Set a 5-second receive timeout */
+    /* Set a 500ms receive timeout */
     if (setsockopt(s, SOL_SOCKET, SO_RCVTIMEO, (const char *)&receiveTimeout, sizeof(receiveTimeout)) == SOCKET_ERROR) goto cleanup;
 
     /* Allocate memory for thread parameters */
@@ -313,6 +344,71 @@ cleanup:
     return ret;
 }
 
+static BOOL send_udp_multicast_of_type(const char *data, int length, ULONG family)
+{
+    IP_ADAPTER_ADDRESSES *adapter_addresses = NULL, *adapter_addr;
+    static const struct in6_addr i_addr_zero;
+    struct addrinfo *multi_address;
+    ULONG bufferSize = 0;
+    LPSOCKADDR sockaddr;
+    BOOL ret = FALSE;
+    const char ttl = 8;
+    ULONG retval;
+    SOCKET s;
+
+   /* Resolve the multicast address */
+    if (family == AF_INET6)
+        multi_address = resolve_address(SEND_ADDRESS_IPV6, SEND_PORT, AF_INET6, SOCK_DGRAM, IPPROTO_UDP);
+    else
+        multi_address = resolve_address(SEND_ADDRESS_IPV4, SEND_PORT, AF_INET, SOCK_DGRAM, IPPROTO_UDP);
+
+    if (multi_address == NULL)
+        return FALSE;
+
+    /* Get size of buffer for adapters */
+    retval = GetAdaptersAddresses(family, 0, NULL, NULL, &bufferSize);
+    if (retval != ERROR_BUFFER_OVERFLOW) goto cleanup;
+
+    adapter_addresses = (IP_ADAPTER_ADDRESSES *) heap_alloc(bufferSize);
+    if (adapter_addresses == NULL) goto cleanup;
+
+    /* Get list of adapters */
+    retval = GetAdaptersAddresses(family, 0, NULL, adapter_addresses, &bufferSize);
+    if (retval != ERROR_SUCCESS) goto cleanup;
+
+    for (adapter_addr = adapter_addresses; adapter_addr != NULL; adapter_addr = adapter_addr->Next)
+    {
+        if (adapter_addr->FirstUnicastAddress == NULL) continue;
+
+        sockaddr = adapter_addr->FirstUnicastAddress->Address.lpSockaddr;
+
+        /* Create a socket and bind to the adapter address */
+        s = socket(family, SOCK_DGRAM, IPPROTO_UDP);
+        if (s == INVALID_SOCKET) continue;
+
+        if (bind(s, sockaddr, adapter_addr->FirstUnicastAddress->Address.iSockaddrLength) == SOCKET_ERROR)
+        {
+            closesocket(s);
+            continue;
+        }
+
+        /* Set the multicast interface and TTL value */
+        setsockopt(s, IPPROTO_IP, IP_MULTICAST_IF, (char *) &i_addr_zero,
+            (family == AF_INET6) ? sizeof(struct in6_addr) : sizeof(struct in_addr));
+        setsockopt(s, IPPROTO_IP, IP_MULTICAST_TTL, &ttl, sizeof(ttl));
+
+        sendto(s, data, length, 0, (SOCKADDR *) multi_address->ai_addr, multi_address->ai_addrlen);
+        closesocket(s);
+    }
+
+    ret = TRUE;
+
+cleanup:
+    freeaddrinfo(multi_address);
+    heap_free(adapter_addresses);
+    return ret;
+}
+
 typedef struct IWSDiscoveryPublisherNotifyImpl {
     IWSDiscoveryPublisherNotify IWSDiscoveryPublisherNotify_iface;
     LONG                  ref;
@@ -372,9 +468,117 @@ static ULONG WINAPI IWSDiscoveryPublisherNotifyImpl_Release(IWSDiscoveryPublishe
     return ref;
 }
 
+static void verify_wsdxml_name(const char *debug_prefix, WSDXML_NAME *name, LPCWSTR uri, LPCWSTR prefix,
+    LPCWSTR local_name)
+{
+    ok(name != NULL, "%s: name == NULL\n", debug_prefix);
+    if (name == NULL) return;
+
+    ok(name->LocalName != NULL && lstrcmpW(name->LocalName, local_name) == 0,
+        "%s: Local name = '%s'\n", debug_prefix, wine_dbgstr_w(name->LocalName));
+
+    ok(name->Space != NULL, "%s: Space == NULL\n", debug_prefix);
+    if (name->Space == NULL) return;
+
+    ok(name->Space->Uri != NULL && lstrcmpW(name->Space->Uri, uri) == 0,
+        "%s: URI == '%s'\n", debug_prefix, wine_dbgstr_w(name->Space->Uri));
+    ok(name->Space->PreferredPrefix != NULL && lstrcmpW(name->Space->PreferredPrefix, prefix) == 0,
+        "%s: Prefix = '%s'\n", debug_prefix, wine_dbgstr_w(name->Space->PreferredPrefix));
+}
+
+static void verify_wsdxml_any_text(const char *debug_prefix, WSDXML_ELEMENT *any, LPCWSTR uri, LPCWSTR prefix,
+    LPCWSTR local_name, LPCWSTR value)
+{
+    WSDXML_TEXT *child;
+
+    ok(any != NULL, "%s: any == NULL\n", debug_prefix);
+    if (any == NULL) return;
+
+    child = (WSDXML_TEXT *) any->FirstChild;
+
+    ok(any->Node.Type == ElementType, "%s: Node type == %d\n", debug_prefix, any->Node.Type);
+    ok(any->Node.Parent == NULL, "%s: Parent == %p\n", debug_prefix, any->Node.Parent);
+    ok(any->Node.Next == NULL, "%s: Next == %p\n", debug_prefix, any->Node.Next);
+    verify_wsdxml_name(debug_prefix, any->Name, uri, prefix, local_name);
+
+    ok(child != NULL, "%s: First child == NULL\n", debug_prefix);
+
+    if (child != NULL)
+    {
+        ok(child->Node.Type == TextType, "%s: Node type == %d\n", debug_prefix, child->Node.Type);
+        ok(child->Node.Parent == any, "%s: Parent == %p\n", debug_prefix, child->Node.Parent);
+        ok(child->Node.Next == NULL, "%s: Next == %p\n", debug_prefix, child->Node.Next);
+
+        if (child->Node.Type == TextType)
+            ok(child->Text != NULL && lstrcmpW(child->Text, value) == 0,
+                "%s: Text == '%s'\n", debug_prefix, wine_dbgstr_w(child->Text));
+    }
+}
+
 static HRESULT WINAPI IWSDiscoveryPublisherNotifyImpl_ProbeHandler(IWSDiscoveryPublisherNotify *This, const WSD_SOAP_MESSAGE *pSoap, IWSDMessageParameters *pMessageParameters)
 {
     trace("IWSDiscoveryPublisherNotifyImpl_ProbeHandler called (%p, %p, %p)\n", This, pSoap, pMessageParameters);
+
+    if (probe_event == NULL)
+    {
+        /* We may have received an unrelated probe on the network */
+        return S_OK;
+    }
+
+    ok(pSoap != NULL, "pSoap == NULL\n");
+    ok(pMessageParameters != NULL, "pMessageParameters == NULL\n");
+
+    if (pSoap != NULL)
+    {
+        static const WCHAR perry[] = {'P','e','r','r','y',0};
+        static const WCHAR extra_info[] = {'E','x','t','r','a','I','n','f','o',0};
+        WSD_PROBE *probe_msg = (WSD_PROBE *) pSoap->Body;
+
+        ok(pSoap->Body != NULL, "pSoap->Body == NULL\n");
+        ok(pSoap->Header.To != NULL && lstrcmpW(pSoap->Header.To, discoveryTo) == 0,
+            "pSoap->Header.To == '%s'\n", wine_dbgstr_w(pSoap->Header.To));
+        ok(pSoap->Header.Action != NULL && lstrcmpW(pSoap->Header.Action, actionProbe) == 0,
+            "pSoap->Header.Action == '%s'\n", wine_dbgstr_w(pSoap->Header.Action));
+
+        ok(pSoap->Header.MessageID != NULL, "pSoap->Header.MessageID == NULL\n");
+
+        /* Ensure the message ID is at least 9 characters long (to skip past the 'urn:uuid:' prefix) */
+        if ((pSoap->Header.MessageID != NULL) && (lstrlenW(pSoap->Header.MessageID) > 9))
+        {
+            UUID uuid;
+            RPC_STATUS ret = UuidFromStringW((LPWSTR)pSoap->Header.MessageID + 9, &uuid);
+
+            trace("Received message with UUID '%s' (expected UUID '%s')\n", wine_dbgstr_guid(&uuid),
+                wine_dbgstr_guid(&probe_message_id));
+
+            /* Check if we've either received a message without a UUID, or the UUID isn't the one we sent. If so,
+               ignore it and wait for another message. */
+            if ((ret != RPC_S_OK) || (UuidEqual(&uuid, &probe_message_id, &ret) == FALSE)) return S_OK;
+        }
+
+        verify_wsdxml_any_text("pSoap->Header.AnyHeaders", pSoap->Header.AnyHeaders, uri_more_tests_no_slash,
+            prefix_grog, perry, extra_info);
+
+        if (probe_msg != NULL)
+        {
+            static const WCHAR lager[] = {'L','a','g','e','r',0};
+            static const WCHAR more_info[] = {'M','o','r','e','I','n','f','o',0};
+
+            ok(probe_msg->Types != NULL, "Probe message Types == NULL\n");
+
+            if (probe_msg->Types != NULL)
+            {
+                verify_wsdxml_name("probe_msg->Types->Element", probe_msg->Types->Element, uri_more_tests_no_slash,
+                    prefix_grog, name_cider);
+                ok(probe_msg->Types->Next == NULL, "probe_msg->Types->Next == %p\n", probe_msg->Types->Next);
+            }
+
+            ok(probe_msg->Scopes == NULL, "Probe message Scopes != NULL\n");
+            verify_wsdxml_any_text("probe_msg->Any", probe_msg->Any, uri_more_tests_no_slash, prefix_grog, lager, more_info);
+        }
+    }
+
+    SetEvent(probe_event);
     return S_OK;
 }
 
@@ -507,18 +711,27 @@ static void Publish_tests(void)
     messageStorage *msgStorage;
     WSADATA wsaData;
     BOOL messageOK, hello_message_seen = FALSE, endpoint_reference_seen = FALSE, app_sequence_seen = FALSE;
-    BOOL metadata_version_seen = FALSE, any_header_seen = FALSE, wine_ns_seen = FALSE;
+    BOOL metadata_version_seen = FALSE, any_header_seen = FALSE, wine_ns_seen = FALSE, body_hello_seen = FALSE;
+    BOOL any_body_seen = FALSE, types_seen = FALSE, xml_namespaces_seen = FALSE, scopes_seen = FALSE;
+    BOOL xaddrs_seen = FALSE;
     int ret, i;
     HRESULT rc;
     ULONG ref;
     char *msg;
-    WSDXML_ELEMENT *header_any_element;
-    WSDXML_NAME header_any_name;
-    WSDXML_NAMESPACE ns;
+    WSDXML_ELEMENT *header_any_element, *body_any_element, *endpoint_any_element, *ref_param_any_element;
+    WSDXML_NAME header_any_name, another_name;
+    WSDXML_NAMESPACE ns, ns2;
     WCHAR header_any_name_text[] = {'B','e','e','r',0};
     static const WCHAR header_any_text[] = {'P','u','b','l','i','s','h','T','e','s','t',0};
+    static const WCHAR body_any_text[] = {'B','o','d','y','T','e','s','t',0};
+    static const WCHAR endpoint_any_text[] = {'E','n','d','P','T','e','s','t',0};
+    static const WCHAR ref_param_any_text[] = {'R','e','f','P','T','e','s','t',0};
     static const WCHAR uri[] = {'h','t','t','p',':','/','/','w','i','n','e','.','t','e','s','t','/',0};
     static const WCHAR prefix[] = {'w','i','n','e',0};
+    static const WCHAR uri3[] = {'h','t','t','p',':','/','/','t','h','i','r','d','.','u','r','l','/',0};
+    WSD_NAME_LIST types_list;
+    WSD_URI_LIST scopes_list, xaddrs_list;
+    unsigned char *probe_uuid_str;
 
     rc = WSDCreateDiscoveryPublisher(NULL, &publisher);
     ok(rc == S_OK, "WSDCreateDiscoveryPublisher(NULL, &publisher) failed: %08x\n", rc);
@@ -590,11 +803,52 @@ static void Publish_tests(void)
     rc = WSDXMLBuildAnyForSingleElement(&header_any_name, header_any_text, &header_any_element);
     ok(rc == S_OK, "WSDXMLBuildAnyForSingleElement failed with %08x\n", rc);
 
+    rc = WSDXMLBuildAnyForSingleElement(&header_any_name, body_any_text, &body_any_element);
+    ok(rc == S_OK, "WSDXMLBuildAnyForSingleElement failed with %08x\n", rc);
+
+    rc = WSDXMLBuildAnyForSingleElement(&header_any_name, endpoint_any_text, &endpoint_any_element);
+    ok(rc == S_OK, "WSDXMLBuildAnyForSingleElement failed with %08x\n", rc);
+
+    rc = WSDXMLBuildAnyForSingleElement(&header_any_name, ref_param_any_text, &ref_param_any_element);
+    ok(rc == S_OK, "WSDXMLBuildAnyForSingleElement failed with %08x\n", rc);
+
+    /* Create types list */
+    ns2.Uri = uri_more_tests;
+    ns2.PreferredPrefix = prefix_grog;
+
+    another_name.LocalName = (WCHAR *) name_cider;
+    another_name.Space = &ns2;
+
+    types_list.Next = malloc(sizeof(WSD_NAME_LIST));
+    types_list.Element = &another_name;
+
+    types_list.Next->Next = NULL;
+    types_list.Next->Element = &header_any_name;
+
+    /* Create scopes and xaddrs lists */
+    scopes_list.Next = malloc(sizeof(WSD_URI_LIST));
+    scopes_list.Element = uri;
+
+    scopes_list.Next->Next = NULL;
+    scopes_list.Next->Element = uri_more_tests;
+
+    xaddrs_list.Next = malloc(sizeof(WSD_URI_LIST));
+    xaddrs_list.Element = uri_more_tests;
+
+    xaddrs_list.Next->Next = NULL;
+    xaddrs_list.Next->Element = uri3;
+
     /* Publish the service */
-    rc = IWSDiscoveryPublisher_PublishEx(publisher, publisherIdW, 1, 1, 1, sequenceIdW, NULL, NULL, NULL,
-        header_any_element, NULL, NULL, NULL, NULL);
+    rc = IWSDiscoveryPublisher_PublishEx(publisher, publisherIdW, 1, 1, 1, sequenceIdW, &types_list, &scopes_list,
+        &xaddrs_list, header_any_element, ref_param_any_element, NULL, endpoint_any_element, body_any_element);
 
     WSDFreeLinkedMemory(header_any_element);
+    WSDFreeLinkedMemory(body_any_element);
+    WSDFreeLinkedMemory(endpoint_any_element);
+    WSDFreeLinkedMemory(ref_param_any_element);
+    free(types_list.Next);
+    free(scopes_list.Next);
+    free(xaddrs_list.Next);
 
     ok(rc == S_OK, "Publish failed: %08x\n", rc);
 
@@ -611,7 +865,10 @@ static void Publish_tests(void)
     /* Verify we've received a message */
     ok(msgStorage->messageCount >= 1, "No messages received\n");
 
-    sprintf(endpointReferenceString, "<wsa:EndpointReference><wsa:Address>%s</wsa:Address></wsa:EndpointReference>", publisherId);
+    sprintf(endpointReferenceString, "<wsa:EndpointReference><wsa:Address>%s</wsa:Address><wsa:ReferenceParameters>"
+        "<wine:Beer>RefPTest</wine:Beer></wsa:ReferenceParameters><wine:Beer>EndPTest</wine:Beer>"
+        "</wsa:EndpointReference>", publisherId);
+
     sprintf(app_sequence_string, "<wsd:AppSequence InstanceId=\"1\" SequenceId=\"%s\" MessageNumber=\"1\"></wsd:AppSequence>",
         sequenceId);
 
@@ -629,8 +886,15 @@ static void Publish_tests(void)
         metadata_version_seen = (strstr(msg, "<wsd:MetadataVersion>1</wsd:MetadataVersion>") != NULL);
         any_header_seen = (strstr(msg, "<wine:Beer>PublishTest</wine:Beer>") != NULL);
         wine_ns_seen = (strstr(msg, "xmlns:wine=\"http://wine.test/\"") != NULL);
+        body_hello_seen = (strstr(msg, "<soap:Body><wsd:Hello") != NULL);
+        any_body_seen = (strstr(msg, "<wine:Beer>BodyTest</wine:Beer>") != NULL);
+        types_seen = (strstr(msg, "<wsd:Types>grog:Cider wine:Beer</wsd:Types>") != NULL);
+        scopes_seen = (strstr(msg, "<wsd:Scopes>http://wine.test/ http://more.tests/</wsd:Scopes>") != NULL);
+        xaddrs_seen = (strstr(msg, "<wsd:XAddrs>http://more.tests/ http://third.url/</wsd:XAddrs>") != NULL);
+        xml_namespaces_seen = (strstr(msg, "xmlns:wine=\"http://wine.test/\" xmlns:grog=\"http://more.tests/\"") != NULL);
         messageOK = hello_message_seen && endpoint_reference_seen && app_sequence_seen && metadata_version_seen &&
-            any_header_seen && wine_ns_seen;
+            any_header_seen && wine_ns_seen && body_hello_seen && any_body_seen && types_seen && xml_namespaces_seen &&
+            scopes_seen && xaddrs_seen;
 
         if (messageOK) break;
     }
@@ -643,18 +907,45 @@ static void Publish_tests(void)
     heap_free(msgStorage);
 
     ok(hello_message_seen == TRUE, "Hello message not received\n");
-    todo_wine ok(endpoint_reference_seen == TRUE, "EndpointReference not received\n");
+    ok(endpoint_reference_seen == TRUE, "EndpointReference not received\n");
     ok(app_sequence_seen == TRUE, "AppSequence not received\n");
-    todo_wine ok(metadata_version_seen == TRUE, "MetadataVersion not received\n");
-    todo_wine ok(messageOK == TRUE, "Hello message metadata not received\n");
+    ok(metadata_version_seen == TRUE, "MetadataVersion not received\n");
+    ok(messageOK == TRUE, "Hello message metadata not received\n");
     ok(any_header_seen == TRUE, "Custom header not received\n");
     ok(wine_ns_seen == TRUE, "Wine namespace not received\n");
+    ok(body_hello_seen == TRUE, "Body and Hello elements not received\n");
+    ok(any_body_seen == TRUE, "Custom body element not received\n");
+    ok(types_seen == TRUE, "Types not received\n");
+    ok(xml_namespaces_seen == TRUE, "XML namespaces not received\n");
+    ok(scopes_seen == TRUE, "Scopes not received\n");
+    ok(xaddrs_seen == TRUE, "XAddrs not received\n");
 
 after_publish_test:
 
     heap_free(publisherIdW);
     heap_free(sequenceIdW);
 
+    /* Test the receiving of a probe message */
+    probe_event = CreateEventW(NULL, TRUE, FALSE, NULL);
+
+    UuidCreate(&probe_message_id);
+    UuidToStringA(&probe_message_id, &probe_uuid_str);
+
+    ok(probe_uuid_str != NULL, "Failed to create UUID for probe message\n");
+
+    if (probe_uuid_str != NULL)
+    {
+        char probe_message[sizeof(testProbeMessage) + 50];
+        sprintf(probe_message, testProbeMessage, probe_uuid_str);
+
+        ok(send_udp_multicast_of_type(probe_message, strlen(probe_message), AF_INET) == TRUE, "Sending Probe message failed\n");
+        todo_wine ok(WaitForSingleObject(probe_event, 2000) == WAIT_OBJECT_0, "Probe message not received\n");
+
+        RpcStringFreeA(&probe_uuid_str);
+    }
+
+    CloseHandle(probe_event);
+
     ref = IWSDiscoveryPublisher_Release(publisher);
     ok(ref == 0, "IWSDiscoveryPublisher_Release() has %d references, should have 0\n", ref);
 
@@ -669,6 +960,143 @@ after_publish_test:
     WSACleanup();
 }
 
+static void UnPublish_tests(void)
+{
+    IWSDiscoveryPublisher *publisher = NULL;
+    IWSDiscoveryPublisherNotify *sink1 = NULL;
+    char endpoint_reference_string[MAX_PATH], app_sequence_string[MAX_PATH];
+    LPWSTR publisherIdW = NULL, sequenceIdW = NULL;
+    messageStorage *msg_storage;
+    WSADATA wsa_data;
+    BOOL message_ok, hello_message_seen = FALSE, endpoint_reference_seen = FALSE, app_sequence_seen = FALSE;
+    BOOL wine_ns_seen = FALSE, body_hello_seen = FALSE, any_body_seen = FALSE;
+    int ret, i;
+    HRESULT rc;
+    ULONG ref;
+    char *msg;
+    WSDXML_ELEMENT *body_any_element;
+    WSDXML_NAME body_any_name;
+    WSDXML_NAMESPACE ns;
+    WCHAR body_any_name_text[] = {'B','e','e','r',0};
+    static const WCHAR body_any_text[] = {'B','o','d','y','T','e','s','t',0};
+    static const WCHAR uri[] = {'h','t','t','p',':','/','/','w','i','n','e','.','t','e','s','t','/',0};
+    static const WCHAR prefix[] = {'w','i','n','e',0};
+
+    rc = WSDCreateDiscoveryPublisher(NULL, &publisher);
+    ok(rc == S_OK, "WSDCreateDiscoveryPublisher(NULL, &publisher) failed: %08x\n", rc);
+    ok(publisher != NULL, "WSDCreateDiscoveryPublisher(NULL, &publisher) failed: publisher == NULL\n");
+
+    rc = IWSDiscoveryPublisher_SetAddressFamily(publisher, WSDAPI_ADDRESSFAMILY_IPV4);
+    ok(rc == S_OK, "IWSDiscoveryPublisher_SetAddressFamily(WSDAPI_ADDRESSFAMILY_IPV4) failed: %08x\n", rc);
+
+    /* Create notification sink */
+    ok(create_discovery_publisher_notify(&sink1) == TRUE, "create_discovery_publisher_notify failed\n");
+    rc = IWSDiscoveryPublisher_RegisterNotificationSink(publisher, sink1);
+    ok(rc == S_OK, "IWSDiscoveryPublisher_RegisterNotificationSink failed: %08x\n", rc);
+
+    /* Set up network listener */
+    publisherIdW = utf8_to_wide(publisherId);
+    if (publisherIdW == NULL) goto after_unpublish_test;
+
+    sequenceIdW = utf8_to_wide(sequenceId);
+    if (sequenceIdW == NULL) goto after_unpublish_test;
+
+    msg_storage = heap_alloc_zero(sizeof(messageStorage));
+    if (msg_storage == NULL) goto after_unpublish_test;
+
+    msg_storage->running = TRUE;
+    InitializeCriticalSection(&msg_storage->criticalSection);
+
+    ret = WSAStartup(MAKEWORD(2, 2), &wsa_data);
+    ok(ret == 0, "WSAStartup failed (ret = %d)\n", ret);
+
+    ret = start_listening_on_all_addresses(msg_storage, AF_INET);
+    ok(ret == TRUE, "Unable to listen on IPv4 addresses (ret == %d)\n", ret);
+
+    /* Create "any" elements for header */
+    ns.Uri = uri;
+    ns.PreferredPrefix = prefix;
+
+    body_any_name.LocalName = body_any_name_text;
+    body_any_name.Space = &ns;
+
+    rc = WSDXMLBuildAnyForSingleElement(&body_any_name, body_any_text, &body_any_element);
+    ok(rc == S_OK, "WSDXMLBuildAnyForSingleElement failed with %08x\n", rc);
+
+    /* Unpublish the service */
+    rc = IWSDiscoveryPublisher_UnPublish(publisher, publisherIdW, 1, 1, sequenceIdW, body_any_element);
+
+    WSDFreeLinkedMemory(body_any_element);
+
+    ok(rc == S_OK, "Unpublish failed: %08x\n", rc);
+
+    /* Wait up to 2 seconds for messages to be received */
+    if (WaitForMultipleObjects(msg_storage->numThreadHandles, msg_storage->threadHandles, TRUE, 2000) == WAIT_TIMEOUT)
+    {
+        /* Wait up to 1 more second for threads to terminate */
+        msg_storage->running = FALSE;
+        WaitForMultipleObjects(msg_storage->numThreadHandles, msg_storage->threadHandles, TRUE, 1000);
+    }
+
+    DeleteCriticalSection(&msg_storage->criticalSection);
+
+    /* Verify we've received a message */
+    ok(msg_storage->messageCount >= 1, "No messages received\n");
+
+    sprintf(endpoint_reference_string, "<wsa:EndpointReference><wsa:Address>%s</wsa:Address></wsa:EndpointReference>",
+        publisherId);
+    sprintf(app_sequence_string, "<wsd:AppSequence InstanceId=\"1\" SequenceId=\"%s\" MessageNumber=\"1\"></wsd:AppSequence>",
+        sequenceId);
+
+    message_ok = FALSE;
+
+    /* Check we're received the correct message */
+    for (i = 0; i < msg_storage->messageCount; i++)
+    {
+        msg = msg_storage->messages[i];
+        message_ok = FALSE;
+
+        hello_message_seen = (strstr(msg, "<wsa:Action>http://schemas.xmlsoap.org/ws/2005/04/discovery/Bye</wsa:Action>") != NULL);
+        endpoint_reference_seen = (strstr(msg, endpoint_reference_string) != NULL);
+        app_sequence_seen = (strstr(msg, app_sequence_string) != NULL);
+        wine_ns_seen = (strstr(msg, "xmlns:wine=\"http://wine.test/\"") != NULL);
+        body_hello_seen = (strstr(msg, "<soap:Body><wsd:Bye") != NULL);
+        any_body_seen = (strstr(msg, "<wine:Beer>BodyTest</wine:Beer>") != NULL);
+        message_ok = hello_message_seen && endpoint_reference_seen && app_sequence_seen && wine_ns_seen &&
+            body_hello_seen && any_body_seen;
+
+        if (message_ok) break;
+    }
+
+    for (i = 0; i < msg_storage->messageCount; i++)
+    {
+        heap_free(msg_storage->messages[i]);
+    }
+
+    heap_free(msg_storage);
+
+    ok(hello_message_seen == TRUE, "Bye message not received\n");
+    ok(endpoint_reference_seen == TRUE, "EndpointReference not received\n");
+    ok(app_sequence_seen == TRUE, "AppSequence not received\n");
+    ok(message_ok == TRUE, "Bye message metadata not received\n");
+    ok(wine_ns_seen == TRUE, "Wine namespace not received\n");
+    ok(body_hello_seen == TRUE, "Body and Bye elements not received\n");
+    ok(any_body_seen == TRUE, "Custom body element not received\n");
+
+after_unpublish_test:
+
+    heap_free(publisherIdW);
+    heap_free(sequenceIdW);
+
+    ref = IWSDiscoveryPublisher_Release(publisher);
+    ok(ref == 0, "IWSDiscoveryPublisher_Release() has %d references, should have 0\n", ref);
+
+    /* Release the sinks */
+    IWSDiscoveryPublisherNotify_Release(sink1);
+
+    WSACleanup();
+}
+
 enum firewall_op
 {
     APP_ADD,
@@ -814,6 +1242,7 @@ START_TEST(discovery)
     CreateDiscoveryPublisher_tests();
     CreateDiscoveryPublisher_XMLContext_tests();
     Publish_tests();
+    UnPublish_tests();
 
     CoUninitialize();
     if (firewall_enabled) set_firewall(APP_REMOVE);
